// chart series pallete
var ec_pal = [
  '#88B04B', '#5B5EA6', '#FF6384', '#FF9F40', '#FFCD56', '#36A2EB',
  '#4BC0C0', '#009B77', '#FF6F61', '#EFC050', '#7FCDCD', '#DFCFBE',
  '#B565A7', '#92A8D1', '#E15D44', '#C3447A', '#955251', '#6B5B95',
  '#45B8AC', '#98B4D4', '#DD4124', '#D65076', '#9B2335', '#55B4B0',
  '#BC243C',  '#F7CAC9'];

// chart global settings
Chart.platform.disableCSSInjection = true;

Chart.defaults.global.defaultFontColor = '#212529';
Chart.defaults.global.maintainAspectRatio = true;
Chart.defaults.global.animation.duration = 0;
Chart.defaults.global.hover.animationDuration = 0;
Chart.defaults.global.responsiveAnimationDuration = 0;
Chart.defaults.global.layout.padding = {left:24,right:24,top:0,bottom:0};

Chart.defaults.global.elements.rectangle.borderWidth = 1;
Chart.defaults.global.elements.line.borderWidth = 2;

Chart.defaults.global.datasets.bar.barPercentage = 0.9;
Chart.defaults.global.datasets.bar.categoryPercentage = 0.9;

Chart.defaults.global.title.display = true;
Chart.defaults.global.title.padding = 20;
Chart.defaults.global.title.fontColor = '#000066';
Chart.defaults.global.title.fontSize = 20;

Chart.defaults.global.tooltips.yAlign = 'bottom';
Chart.defaults.global.tooltips.xAlign = 'center';
Chart.defaults.global.tooltips.cornerRadius = 3;

Chart.defaults.global.legend.position = 'bottom';
Chart.defaults.global.legend.labels.boxWidth = 15;
Chart.defaults.global.legend.labels.padding = 8;
Chart.defaults.global.legend.labels.fontSize = 16;
Chart.defaults.global.legend.labels.filter = function(legendItem, data) {
    if (data.datasets.length > 0) {
      if (['pie', 'doughnut', 'polarArea'].includes(data.datasets[0]._meta[0].type))  // filter zero value series
        {if (data.datasets[0].data[legendItem.index]) {return true;} else {return false;}}
      else
        {return (legendItem.text != '');} // filter datasets with an empty label
    };
  }


// create a new chart
function fncNewChart(chartCanvas, chartType, chartTitle, chartLabels, chartDatasets, chartTimeScale, chartLocale, chartPrec, chartValLbls, chartLegendReverse, yAutoScale) {
  var chartCanvas = document.getElementById(chartCanvas);
  var chartCtx = chartCanvas.getContext("2d");
  var gh = chartCanvas.parentElement.offsetHeight;

  // prepare general options
  var chartOptions = {
      legend: {},
      tooltips: {
        callbacks: {
          label: function(tooltipItem, data) {
            if (['pie', 'doughnut', 'polarArea'].includes(chartType))
            {
              var label = data.labels[tooltipItem.index] || '';
              if (label) {label = label.trim() + ': ';}
              return ' ' + label + data.datasets[tooltipItem.datasetIndex].data[tooltipItem.index].toLocaleString(chartLocale, {minimumFractionDigits: chartPrec, maximumFractionDigits: chartPrec});
            }
            else
            {
              var label = data.datasets[tooltipItem.datasetIndex].label || '';
              if (label) {if (label.trim() == '/') {label = '';} else {label = label.trim() + ': ';}}
              if (['horizontalBar', 'horizontalBar-stacked'].includes(chartType))
                {return ' ' + label + tooltipItem.xLabel.toLocaleString(chartLocale, {minimumFractionDigits: chartPrec, maximumFractionDigits: chartPrec});}
              else
                {return ' ' + label + tooltipItem.yLabel.toLocaleString(chartLocale, {minimumFractionDigits: chartPrec, maximumFractionDigits: chartPrec});}
            }}}},
      title: {text: chartTitle},
      plugins: {
          labels: [
            {
              //arc: true,
              position: 'outside',
              textMargin: 5,
              fontSize: 14,
              render: function (args) {
                if (args.dataset.order < 0) {return '';} else {return args.value.toLocaleString(chartLocale, {minimumFractionDigits: chartPrec, maximumFractionDigits: chartPrec});}
              }
            }
            ]}
    };

  if (chartValLbls == false) {
    chartOptions.plugins.labels[0].render = 'image';
  }
  if (chartLegendReverse == true) {
    chartOptions.legend.reverse = true;
  }

  // prepare by chart type
  var i, j, k, gradient;
  j = 0;

  if (['pie', 'doughnut', 'polarArea'].includes(chartType))
  {
    for (i = 0; i < chartDatasets.length; i++)
    {
      chartDatasets[i].borderWidth = 1;
      chartDatasets[i].borderAlign = 'inner';
      chartDatasets[i].backgroundColor = [];
      chartDatasets[i].borderColor = [];
      chartDatasets[i].hoverBorderWidth = 2;
      chartDatasets[i].hoverBackgroundColor = [];
      for (k = 0; k < chartDatasets[i].data.length; k++)
      {
        if (chartLabels[k] != '') chartLabels[k] += '   ';
        gradient = chartCtx.createLinearGradient(0, 0, 0, gh);
        gradient.addColorStop(0, ec_pal[j]);
        gradient.addColorStop(1, ec_pal[j]+'77');
        chartDatasets[i].backgroundColor.push(gradient);
        chartDatasets[i].hoverBackgroundColor.push(ec_pal[j]+'e9');
        chartDatasets[i].borderColor.push(ec_pal[j]+'77');
        j++; if (j >= ec_pal.length) j = 0;
      }
    }

    if (['polarArea'].includes(chartType)) {
      chartOptions.scale = {
          ticks: {beginAtZero: !yAutoScale,
            callback: function(value, index, values) {return value.toLocaleString(chartLocale);}}
        };}
  }
  else
  {
    for (i = 0; i < chartDatasets.length; i++)
    {
      if (chartDatasets[i].label != '') chartDatasets[i].label += '   ';
      if (['horizontalBar', 'horizontalBar-stacked'].includes(chartType))
        gradient = chartCtx.createLinearGradient(gh, 0, 0, 0);
      else
        gradient = chartCtx.createLinearGradient(0, 0, 0, gh);
      gradient.addColorStop(0, ec_pal[j]);
      gradient.addColorStop(1, ec_pal[j]+'33');

      chartDatasets[i].backgroundColor = gradient;
      chartDatasets[i].borderColor = ec_pal[j];
      chartDatasets[i].hoverBackgroundColor = ec_pal[j]+'e9';

      if (chartType == 'radar')
      {
        chartDatasets[i].pointBackgroundColor = ec_pal[j];
        chartDatasets[i].pointRadius = 2;
        chartDatasets[i].pointHoverRadius = 3;
      }
      else
      if (((chartDatasets[i].type) && (chartDatasets[i].type == 'line')) || ((chartType == 'line')))
      {
        chartDatasets[i].pointBackgroundColor = ec_pal[j];
        if (!chartDatasets[i].fill) chartDatasets[i].fill = false;
      }
      j++; if (j >= ec_pal.length) j = 0;
    }

    if (['radar'].includes(chartType))
    {
      chartOptions.scale = {
          ticks: {beginAtZero: !yAutoScale,
            callback: function(value, index, values) {return value.toLocaleString(chartLocale);}}
        };
    }
    else
    {
      rptYAxes = [{
          ticks: {beginAtZero: !yAutoScale, padding:5, callback: function(value, index, values) {return value.toLocaleString(chartLocale, {minimumFractionDigits: chartPrec, maximumFractionDigits: chartPrec});}},
          gridLines: {color: '#ddd', borderDash: [2], drawBorder: false, tickMarkLength: 4}
        }];
      rptXAxes = [{
          offset: true,
          ticks: {padding:4},
          gridLines: {color: '#ddd', borderDash: [2], drawBorder: false, tickMarkLength: 4}
        }];
      if (chartTimeScale)
      {
        rptXAxes[0].type = 'time';
        rptXAxes[0].distribution = 'linear';
        rptXAxes[0].bounds = 'data';
        rptXAxes[0].time = {minUnit: 'day', displayFormats: {day: 'YYYY-MM-DD', month: 'YYYY-MM', quarter: 'YYYY-[Q]Q'}};

        if (chartLabels.length > 0) {
          if ((chartLabels[0].trim().length > 2) && (chartLabels[0].charAt(2) == ':'))
          {
            if (chartLabels[0].trim().length > 6) {rptXAxes[0].time.minUnit = 'second';}
            else if (chartLabels[0].trim().length > 3) {rptXAxes[0].time.minUnit = 'minute';}
            else {rptXAxes[0].time.minUnit = 'hour';}
          }
          else
          {
            if (chartLabels[0].trim().length <= 4) {rptXAxes[0].time.unit = 'year'; rptXAxes[0].time.min = chartLabels[0].trim() + '-01-01';}
            else if (chartLabels[0].trim().length <= 7) {rptXAxes[0].time.unit = 'month';}
          }
        }

      }

      chartOptions.scales = {yAxes: rptYAxes, xAxes: rptXAxes};
    }
  }

  if (chartType == 'area') {
    chartType = 'line';
  }
  else if (chartType == 'area-stacked') {
    chartType = 'line';
    chartOptions.scales.yAxes[0].stacked = true;
  }
  else if (chartType == 'horizontalBar') {
    asw = chartOptions.scales.xAxes[0];
    chartOptions.scales.xAxes[0] = chartOptions.scales.yAxes[0];
    chartOptions.scales.yAxes[0] = asw;
  }
  else if (chartType == 'horizontalBar-stacked') {
    chartType = 'horizontalBar';
    chartOptions.scales.yAxes[0].stacked = true;
    chartOptions.scales.xAxes[0].stacked = true;
    asw = chartOptions.scales.xAxes[0];
    chartOptions.scales.xAxes[0] = chartOptions.scales.yAxes[0];
    chartOptions.scales.yAxes[0] = asw;
  }
  else if (chartType == 'bar-stacked') {
    chartType = 'bar';
    chartOptions.scales.yAxes[0].stacked = true;
    chartOptions.scales.xAxes[0].stacked = true;
  }

  // create a new chart
  var resChart = new Chart(chartCanvas,
    {
      type: chartType,
      options: chartOptions,
      data: {
        labels: chartLabels,
        datasets: chartDatasets
      }
    });

  return resChart;
}